/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.gui.QueryView;

import java.util.Collection;
import java.util.Vector;

import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.gui.quickview.ExpressViewConfigurator;
import ca.uvic.csr.shrimp.gui.quickview.QuickViewAction;

/**
 * An action that configures the view to show the results
 * of a query, that is nodes of interest and their neighbours. 
 * 
 * @author Rob Lintern
 */
public class QueryViewAction extends QuickViewAction {

	/**
	 * Don't use this constructor.  It is used to instantiate this class using reflection.
	 */
	public QueryViewAction() {
		super();
	}
	
    public QueryViewAction(String actionName, ShrimpProject project, Collection nodeTypes, 
    		Collection arcTypes, String layoutMode, String labelMode, int incomingLevels, int outgoingLevels) {
        super(actionName, project);

        config.setNodeTypesOfInterest(nodeTypes);
        config.setArcTypesOfInterest(arcTypes);
        config.setLayoutMode(layoutMode);
        config.setLabelMode(labelMode);
        config.setCprels(new String[0]);
        getQueryViewConfigurator().setLevels(incomingLevels, outgoingLevels);
    }
    
    protected ExpressViewConfigurator createExpressViewConfigurator(ShrimpProject project) {
    	return new QueryViewConfigurator(project);
    }
    
    private QueryViewConfigurator getQueryViewConfigurator() {
    	return (QueryViewConfigurator) config;
    }
    
    public void startAction() {
        if (!getSrcArtifacts().isEmpty()) {
    	    super.startAction();
        }
    }   

    public void setSrcExternalArtifactIDs(Collection srcArtifactExternalIDs) {
    	getQueryViewConfigurator().setSrcExternalArtifactIDs(srcArtifactExternalIDs);
    }
    
    /**
     * Overridden to verify if any src external artifact IDs have been set.
     */
    public boolean isValid() {
    	return super.isValid() && (getQueryViewConfigurator().getSrcExternalArtifactIDs().size() > 0);
    }

    public void applyDataFilters() {
        config.applyDataFilters();        
    }
    
    public Vector getSrcArtifacts() {
        return getQueryViewConfigurator().getSrcArtifacts();
    }

}
