/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */

package ca.uvic.csr.shrimp.usercontrols;

import java.util.Vector;

import ca.uvic.csr.shrimp.SelectorBeanConstants;
import ca.uvic.csr.shrimp.DisplayBean.DisplayConstants;
import ca.uvic.csr.shrimp.DisplayBean.ShrimpArc;
import ca.uvic.csr.shrimp.DisplayBean.ShrimpNode;
import ca.uvic.csr.shrimp.SelectorBean.SelectorBean;
import ca.uvic.csr.shrimp.ShrimpApplication.BeanNotFoundException;
import ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool;

/**
 * Causes the shrimp view to navigate to the source of an arc
 * @author Rob Lintern, Chris Callendar
 */
public class NavigateToArcSourceAdapter extends DefaultToolAction {
	
	public static final String ACTION_NAME = "Go To Source";
	
	public NavigateToArcSourceAdapter (ShrimpTool tool) {
		super(ACTION_NAME, tool);
	    mustStartAndStop = false;
	}
	
	/**
	 * @see ca.uvic.csr.shrimp.usercontrols.UserAction#startAction()
	 */
	public void startAction() {
		try {
			SelectorBean selectorBean = (SelectorBean) tool.getBean(ShrimpTool.SELECTOR_BEAN);
			Vector selectedArcs = (Vector) selectorBean.getSelected(SelectorBeanConstants.SELECTED_ARCS);
			if (selectedArcs.size() == 1) {
				ShrimpArc arc = (ShrimpArc) selectedArcs.firstElement();
				ShrimpNode srcNode = arc.getSrcNode();
				
				Object oldTarget = selectorBean.getSelected(SelectorBeanConstants.TARGET_OBJECT);
				selectorBean.setSelected (SelectorBeanConstants.TARGET_OBJECT, srcNode); 
				Object oldZoomMode = selectorBean.getSelected(DisplayConstants.ZOOM_MODE);
				selectorBean.setSelected(DisplayConstants.ZOOM_MODE, DisplayConstants.MAGNIFY);
				
				MagnifyInAdapter tempMIA = new MagnifyInAdapter(tool);
				tempMIA.startAction();
				if (oldTarget != null)
					selectorBean.setSelected(SelectorBeanConstants.TARGET_OBJECT, oldTarget);
				selectorBean.setSelected(DisplayConstants.ZOOM_MODE, oldZoomMode);
			}
		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		}		
	}

}
